from .helpers import *

class TestClientWorkspaces(ClientTestCase):

    def test_workspaces_find_all(self):
        res = { "data": [{ "id": 1337, "name": "My Favorite Workspace" }]}
        responses.add(GET, 'http://app/workspaces', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.workspaces.find_all(), res['data'])

    def test_workspaces_update(self):
        req = { "data": { "name": "Everyone's Favorite Workspace" } }
        res = { "data": { "id": 1337, "name": "Everyone's Favorite Workspace" } }
        responses.add(PUT, 'http://app/workspaces/1337', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.workspaces.update(1337, req['data']), res['data'])
        self.assertEqual(json.loads(responses.calls[0].request.body), req)

    def test_workspaces_typeahead(self):
        req = { "data": { "type": "user", "query": "Greg" }}
        res = { "data": [{ "id": 999, "name": "Greg Sanchez" }]}
        responses.add(GET, 'http://app/workspaces/1337/typeahead?type=user&query=Greg', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.workspaces.typeahead(1337, req['data']), res['data'])




class _Projects:
    """A _project_ represents a prioritized list of tasks in Asana. It exists in a
    single workspace or organization and is accessible to a subset of users in
    that workspace or organization, depending on its permissions.
    
    Projects in organizations are shared with a single team. You cannot currently
    change the team of a project via the API. Non-organization workspaces do not
    have teams and so you should not specify the team of project in a
    regular workspace.
    """

    def __init__(self, client=None):
        self.client = client
  
    def create(self, params={}, **options): 
        """Creates a new project in a workspace or team.
        
        Every project is required to be created in a specific workspace or
        organization, and this cannot be changed once set. Note that you can use
        the `workspace` parameter regardless of whether or not it is an
        organization.
        
        If the workspace for your project _is_ an organization, you must also
        supply a `team` to share the project with.
        
        Returns the full record of the newly created project.

        Parameters
        ----------
        [data] : {Object} Data for the request
          - workspace : {Id} The workspace or organization to create the project in.
          - [team] : {Id} If creating in an organization, the specific team to create the
          project in.
        """
        return self.client.post("/projects", params, **options)
        
    def create_in_workspace(self, workspace, params={}, **options): 
        """If the workspace for your project _is_ an organization, you must also
        supply a `team` to share the project with.
        
        Returns the full record of the newly created project.

        Parameters
        ----------
        workspace : {Id} The workspace or organization to create the project in.
        [data] : {Object} Data for the request
        """
        path = "/workspaces/%d/projects" % (workspace)
        return self.client.post(path, params, **options)
        
    def create_in_team(self, team, params={}, **options): 
        """Creates a project shared with the given team.
        
        Returns the full record of the newly created project.

        Parameters
        ----------
        team : {Id} The team to create the project in.
        [data] : {Object} Data for the request
        """
        path = "/teams/%d/projects" % (team)
        return self.client.post(path, params, **options)
        
    def find_by_id(self, project, params={}, **options): 
        """Returns the complete project record for a single project.

        Parameters
        ----------
        project : {Id} The project to get.
        [params] : {Object} Parameters for the request
        """
        path = "/projects/%d" % (project)
        return self.client.get(path, params, **options)
        
    def update(self, project, params={}, **options): 
        """A specific, existing project can be updated by making a PUT request on the
        URL for that project. Only the fields provided in the `data` block will be
        updated; any unspecified fields will remain unchanged.
        
        When using this method, it is best to specify only those fields you wish
        to change, or else you may overwrite changes made by another user since
        you last retrieved the task.
        
        Returns the complete updated project record.

        Parameters
        ----------
        project : {Id} The project to update.
        [data] : {Object} Data for the request
        """
        path = "/projects/%d" % (project)
        return self.client.put(path, params, **options)
        
    def delete(self, project, params={}, **options): 
        """A specific, existing project can be deleted by making a DELETE request
        on the URL for that project.
        
        Returns an empty data record.

        Parameters
        ----------
        project : {Id} The project to delete.
        """
        path = "/projects/%d" % (project)
        return self.client.delete(path, params, **options)
        
    def find_all(self, params={}, **options): 
        """Returns the compact project records for some filtered set of projects.
        Use one or more of the parameters provided to filter the projects returned.

        Parameters
        ----------
        [params] : {Object} Parameters for the request
          - [workspace] : {Id} The workspace or organization to filter projects on.
          - [team] : {Id} The team to filter projects on.
          - [archived] : {Boolean} Only return projects whose `archived` field takes on the value of
          this parameter.
        """
        return self.client.get_collection("/projects", params, **options)
        
    def find_by_workspace(self, workspace, params={}, **options): 
        """Returns the compact project records for all projects in the workspace.

        Parameters
        ----------
        workspace : {Id} The workspace or organization to find projects in.
        [params] : {Object} Parameters for the request
          - [archived] : {Boolean} Only return projects whose `archived` field takes on the value of
          this parameter.
        """
        path = "/workspaces/%d/projects" % (workspace)
        return self.client.get_collection(path, params, **options)
        
    def find_by_team(self, team, params={}, **options): 
        """Returns the compact project records for all projects in the team.

        Parameters
        ----------
        team : {Id} The team to find projects in.
        [params] : {Object} Parameters for the request
          - [archived] : {Boolean} Only return projects whose `archived` field takes on the value of
          this parameter.
        """
        path = "/teams/%d/projects" % (team)
        return self.client.get_collection(path, params, **options)
        
    def sections(self, project, params={}, **options): 
        """Returns compact records for all sections in the specified project.

        Parameters
        ----------
        project : {Id} The project to get sections from.
        [params] : {Object} Parameters for the request
        """
        path = "/projects/%d/sections" % (project)
        return self.client.get_collection(path, params, **options)
        
    def get_tasks_in_project(self, project, params={}, **options): 
        """Returns the compact task records for all tasks within the given project,
        ordered by their priority within the project. Tasks can exist in more than one project at a time.

        Parameters
        ----------
        project : {Id} The project in which to search for tasks.
        [params] : {Object} Parameters for the request
        """
        path = "/projects/%d/tasks" % (project)
        return self.client.get_collection(path, params, **options)
        



from .helpers import *

class TestClientTeams(ClientTestCase):

    def test_teams_find_by_organization(self):
        res = {
            "data": [
                { "id": 5832, "name": "Atlanta Braves" },
                { "id": 15923, "name": "New York Yankees" }
            ]
        }
        responses.add(GET, 'http://app/organizations/13523/teams', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.teams.find_by_organization(13523), res['data'])



from .helpers import *

class TestClientStories(ClientTestCase):

    def test_stories_find_by_task(self):
        res = {
            "data": [
                {
                    "created_at": "2011-12-21T23:23:01.259Z",
                    "created_by": { "id": 5678, "name": "Greg Sanchez" },
                    "id": 2001,
                    "text": "added to Things To Buy",
                    "type": "system"
                },
                {
                    "created_at": "2012-01-02T21:32:40.112Z",
                    "created_by": { "id": 1234, "name": "Tim Bizarro" },
                    "id": 2002,
                    "text": "Again? Wow, we really go through this stuff fast.",
                    "type": "comment"
                }
            ]
        }
        responses.add(GET, 'http://app/tasks/1001/stories', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.stories.find_by_task(1001), res['data'])

    def test_stories_find_by_id(self):
        res = {
            "data": {
                "created_at": "2012-02-22T02:06:58.147Z",
                "created_by": { "id": 1123, "name": "Mittens" },
                "id": 2001,
                "source": "web",
                "target": { "id": 1234, "name": "Buy catnip" },
                "text": "Yes, please!",
                "type": "comment"
            }
        }
        responses.add(GET, 'http://app/stories/2001', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.stories.find_by_id(2001), res['data'])

    def test_stories_create_on_task(self):
        req = { "data": { "text": "This is a very nice comment." } }
        res = {
            "data": {
                "created_at": "2011-12-21T23:23:01.259Z",
                "created_by": { "id": 5678, "name": "Greg Sanchez" },
                "id": 2001,
                "source": "api",
                "target": { "id": 1001, "name": "Buy catnip" },
                "text": "This is a very nice comment.",
                "type": "comment"
            }
        }
        responses.add(POST, 'http://app/tasks/1001/stories', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.stories.create_on_task(1001, req['data']), res['data'])
        self.assertEqual(json.loads(responses.calls[0].request.body), req)



from .helpers import *

class TestClientProjects(ClientTestCase):
    def test_projects_create(self):
        req = {
            "data": {
                "name": "Things to Buy",
                "notes": "These are things we want to purchase.",
                "workspace": 14916
            }
        }
        res = {
            "data": {
                "id": 1331,
                "name": "Things to Buy",
                "notes": "These are things we want to purchase."
            }
        }
        responses.add(POST, 'http://app/projects', status=201, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.projects.create(req['data']), res['data'])
        self.assertEqual(json.loads(responses.calls[0].request.body), req)

    def test_projects_find_by_id(self):
        res = {
            "data": {
                "id": 1331,
                "name": "Things to Buy",
                "notes": "These are things we want to purchase."
            }
        }
        responses.add(GET, 'http://app/projects/1331', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.projects.find_by_id(1331), res['data'])

    def test_projects_update(self):
        req = {
            "data": {
                "notes": "These are things we NEED to purchase."
            }
        }
        res = {
            "data": {
                "id": 1331,
                "name": "Things to Buy",
                "notes": "These are things we NEED to purchase."
            }
        }
        responses.add(PUT, 'http://app/projects/1331', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.projects.update(1331, req['data']), res['data'])
        self.assertEqual(json.loads(responses.calls[0].request.body), req)

    def test_projects_delete(self):
        res = { "data": {} }
        responses.add(DELETE, 'http://app/projects/1331', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.projects.delete(1331), res['data'])

    def test_projects_find_by_workspace(self):
        res = {
            "data": [
                { "id": 1331, "name": "Things to buy" },
                { "id": 14641, "name": "Cat Stuff" }
            ]
        }
        responses.add(GET, 'http://app/workspaces/14916/projects', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.projects.find_by_workspace(14916), res['data'])

    def test_projects_find_all(self):
        res = {
            "data": [
                { "id": 1331, "name": "Things to buy" },
                { "id": 14641, "name": "Cat Stuff" }
            ]
        }
        responses.add(GET, 'http://app/projects', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.projects.find_all(), res['data'])



from .helpers import *
from mock import patch, call

class TestClient(ClientTestCase):

    def test_version_values(self):
        values = self.client._versionValues()
        self.assertEqual('Python', values['language'])
        self.assertRegexpMatches(values['version'], r'[0-9]+[.][0-9]+[.][0-9]+')
        self.assertSetEqual(set([
            'language', 'version', 'language_version', 'os', 'os_version'
        ]), set(values.keys()))

    def test_version_header(self):
        self.assertRegexpMatches(
            self.client._versionHeader(), r'language=Python')

    def test_users_me_not_authorized(self):
        res = {
            "errors": [
                { "message": "Not Authorized" }
            ]
        }
        responses.add(GET, 'http://app/users/me', status=401, body=json.dumps(res), match_querystring=True)
        self.assertRaises(asana.error.NoAuthorizationError, self.client.users.me)

    def test_tasks_find_all_invalid_request(self):
        res = {
            "errors": [
                { "message": "workspace: Missing input" }
            ]
        }
        responses.add(GET, 'http://app/tasks', status=400, body=json.dumps(res), match_querystring=True)
        self.assertRaises(asana.error.InvalidRequestError, self.client.tasks.find_all)

    def test_users_me_server_error(self):
        res = {
            "errors": [
                {
                    "message": "Server Error",
                    "phrase": "6 sad squid snuggle softly"
                }
            ]
        }
        responses.add(GET, 'http://app/users/me', status=500, body=json.dumps(res), match_querystring=True)
        self.assertRaises(asana.error.ServerError, self.client.users.me, max_retries=0)

    def test_users_me_unfriendly_server_error(self):
        res = "Crappy Response"
        responses.add(GET, 'http://app/users/me', status=504, body=res, match_querystring=True)
        self.assertRaises(asana.error.ServerError, self.client.users.me, max_retries=0)

    def test_users_find_by_id_not_found(self):
        res = {
            "errors": [
                { "message": "user: Unknown object: 1234124312341" }
            ]
        }
        responses.add(GET, 'http://app/users/1234', status=404, body=json.dumps(res), match_querystring=True)
        self.assertRaises(asana.error.NotFoundError, self.client.users.find_by_id, (1234))

    def test_users_find_by_id_forbidden(self):
        res = {
            "errors": [
                { "message": "user: Forbidden" }
            ]
        }
        responses.add(GET, 'http://app/users/1234', status=403, body=json.dumps(res), match_querystring=True)
        self.assertRaises(asana.error.ForbiddenError, self.client.users.find_by_id, (1234))

    def test_option_pretty(self):
        res = {
            "data": { "email": "sanchez@...", "id": 999, "name": "Greg Sanchez" }
        }
        # responses.add(GET, 'http://app/users/me?opt_pretty', status=200, body=json.dumps(res), match_querystring=True)
        responses.add(GET, 'http://app/users/me?opt_pretty=true', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.users.me(pretty=True), res['data']) 

    def test_option_fields(self):
        res = {
            "data": { "name": "Make a list", "notes": "Check it twice!", "id": 1224 }
        }
        responses.add(GET, 'http://app/tasks/1224?opt_fields=name%2Cnotes', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.tasks.find_by_id(1224, fields=['name','notes']), res['data'])

    def test_option_expand(self):
        req = {
            'data': { 'assignee': 1234 },
            'options': { 'expand' : ['projects'] }
        }
        res = {
            "data": {
                "id": 1001,
                "projects": [
                    {
                        "archived": false,
                        "created_at": "",
                        "followers": [],
                        "modified_at": "",
                        "notes": "",
                        "id": 1331,
                        "name": "Things to buy"
                    }
                ]
                # ...
            }
        }
        responses.add(PUT, 'http://app/tasks/1001', status=200, body=json.dumps(res), match_querystring=True)
        # -d "assignee=1234" \
        # -d "options.expand=%2Aprojects%2A"
        self.assertEqual(self.client.tasks.update(1001, req['data'], expand=['projects']), res['data'])
        self.assertEqual(json.loads(responses.calls[0].request.body), req)

    def test_pagination(self):
        res = {
            "data": [
                { "id": 1000, "name": "Task 1" }
            ],
            "next_page": {
                "offset": "yJ0eXAiOiJKV1QiLCJhbGciOiJIRzI1NiJ9",
                "path": "/tasks?project=1337&limit=5&offset=yJ0eXAiOiJKV1QiLCJhbGciOiJIRzI1NiJ9",
                "uri": "https://app.asana.com/api/1.0/tasks?project=1337&limit=5&offset=yJ0eXAiOiJKV1QiLCJhbGciOiJIRzI1NiJ9"
            }
        }
        responses.add(GET, 'http://app/projects/1337/tasks?limit=5&offset=eyJ0eXAiOJiKV1iQLCJhbGciOiJIUzI1NiJ9', status=200, body=json.dumps(res), match_querystring=True)

        self.assertEqual(self.client.tasks.find_by_project(1337, { 'limit': 5, 'offset': 'eyJ0eXAiOJiKV1iQLCJhbGciOiJIUzI1NiJ9'}), res['data'])

    @unittest.skip("iterator_type='pages' disabled")
    def test_page_iterator_item_limit_lt_items(self):
        responses.add(GET, 'http://app/projects/1337/tasks?limit=2', status=200, body=json.dumps({ 'data': ['a', 'b'], 'next_page': { 'offset': 'a', 'path': '/projects/1337/tasks?limit=2&offset=a' } }), match_querystring=True)
        responses.add(GET, 'http://app/projects/1337/tasks?limit=2&offset=a', status=200, body=json.dumps({ 'data': ['c'], 'next_page': null }), match_querystring=True)

        iterator = self.client.tasks.find_by_project(1337, item_limit=2, page_size=2, iterator_type='pages')
        self.assertEqual(next(iterator), ['a', 'b'])
        self.assertRaises(StopIteration, next, (iterator))

    @unittest.skip("iterator_type='pages' disabled")
    def test_page_iterator_item_limit_eq_items(self):
        responses.add(GET, 'http://app/projects/1337/tasks?limit=2', status=200, body=json.dumps({ 'data': ['a', 'b'], 'next_page': { 'offset': 'a', 'path': '/projects/1337/tasks?limit=2&offset=a' } }), match_querystring=True)
        responses.add(GET, 'http://app/projects/1337/tasks?limit=1&offset=a', status=200, body=json.dumps({ 'data': ['c'], 'next_page': null }), match_querystring=True)

        iterator = self.client.tasks.find_by_project(1337, item_limit=3, page_size=2, iterator_type='pages')
        self.assertEqual(next(iterator), ['a', 'b'])
        self.assertEqual(next(iterator), ['c'])
        self.assertRaises(StopIteration, next, (iterator))

    @unittest.skip("iterator_type='pages' disabled")
    def test_page_iterator_item_limit_gt_items(self):
        responses.add(GET, 'http://app/projects/1337/tasks?limit=2', status=200, body=json.dumps({ 'data': ['a', 'b'], 'next_page': { 'offset': 'a', 'path': '/projects/1337/tasks?limit=2&offset=a' } }), match_querystring=True)
        responses.add(GET, 'http://app/projects/1337/tasks?limit=2&offset=a', status=200, body=json.dumps({ 'data': ['c'], 'next_page': null }), match_querystring=True)

        iterator = self.client.tasks.find_by_project(1337, item_limit=4, page_size=2, iterator_type='pages')
        self.assertEqual(next(iterator), ['a', 'b'])
        self.assertEqual(next(iterator), ['c'])
        self.assertRaises(StopIteration, next, (iterator))

    def test_item_iterator_item_limit_lt_items(self):
        responses.add(GET, 'http://app/projects/1337/tasks?limit=2', status=200, body=json.dumps({ 'data': ['a', 'b'], 'next_page': { 'offset': 'a', 'path': '/projects/1337/tasks?limit=2&offset=a' } }), match_querystring=True)

        iterator = self.client.tasks.find_by_project(1337, item_limit=2, page_size=2, iterator_type='items')
        self.assertEqual(next(iterator), 'a')
        self.assertEqual(next(iterator), 'b')
        self.assertRaises(StopIteration, next, (iterator))

    def test_item_iterator_item_limit_eq_items(self):
        responses.add(GET, 'http://app/projects/1337/tasks?limit=2', status=200, body=json.dumps({ 'data': ['a', 'b'], 'next_page': { 'offset': 'a', 'path': '/projects/1337/tasks?limit=2&offset=a' } }), match_querystring=True)
        responses.add(GET, 'http://app/projects/1337/tasks?limit=1&offset=a', status=200, body=json.dumps({ 'data': ['c'], 'next_page': null }), match_querystring=True)

        iterator = self.client.tasks.find_by_project(1337, item_limit=3, page_size=2, iterator_type='items')
        self.assertEqual(next(iterator), 'a')
        self.assertEqual(next(iterator), 'b')
        self.assertEqual(next(iterator), 'c')
        self.assertRaises(StopIteration, next, (iterator))

    def test_item_iterator_item_limit_gt_items(self):
        responses.add(GET, 'http://app/projects/1337/tasks?limit=2', status=200, body=json.dumps({ 'data': ['a', 'b'], 'next_page': { 'offset': 'a', 'path': '/projects/1337/tasks?limit=2&offset=a' } }), match_querystring=True)
        responses.add(GET, 'http://app/projects/1337/tasks?limit=2&offset=a', status=200, body=json.dumps({ 'data': ['c'], 'next_page': null }), match_querystring=True)

        iterator = self.client.tasks.find_by_project(1337, item_limit=4, page_size=2, iterator_type='items')
        self.assertEqual(next(iterator), 'a')
        self.assertEqual(next(iterator), 'b')
        self.assertEqual(next(iterator), 'c')
        self.assertRaises(StopIteration, next, (iterator))

    def test_item_iterator_preserve_opt_fields(self):
        responses.add(GET, 'http://app/projects/1337/tasks?limit=2&opt_fields=foo', status=200, body=json.dumps({ 'data': ['a', 'b'], 'next_page': { 'offset': 'a', 'path': '/projects/1337/tasks?limit=2&offset=a' } }), match_querystring=True)
        responses.add(GET, 'http://app/projects/1337/tasks?limit=1&opt_fields=foo&offset=a', status=200, body=json.dumps({ 'data': ['c'], 'next_page': null }), match_querystring=True)

        iterator = self.client.tasks.find_by_project(1337, item_limit=3, page_size=2, fields=['foo'], iterator_type='items')
        self.assertEqual(next(iterator), 'a')
        self.assertEqual(next(iterator), 'b')
        self.assertEqual(next(iterator), 'c')
        self.assertRaises(StopIteration, next, (iterator))

    @patch('time.sleep')
    def test_rate_limiting(self, time_sleep):
        res = [
            (429, { 'Retry-After': '10' }, '{}'),
            (200, {}, json.dumps({ 'data': 'me' }))
        ]
        responses.add_callback(responses.GET, 'http://app/users/me', callback=lambda r: res.pop(0), content_type='application/json')

        self.assertEqual(self.client.users.me(), 'me')
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(time_sleep.mock_calls, [call(10.0)])

    @patch('time.sleep')
    def test_rate_limited_twice(self, time_sleep):
        res = [
            (429, { 'Retry-After': '10' }, '{}'),
            (429, { 'Retry-After': '1' }, '{}'),
            (200, {}, json.dumps({ 'data': 'me' }))
        ]
        responses.add_callback(responses.GET, 'http://app/users/me', callback=lambda r: res.pop(0), content_type='application/json')

        self.assertEqual(self.client.users.me(), 'me')
        self.assertEqual(len(responses.calls), 3)
        self.assertEqual(time_sleep.mock_calls, [call(10.0), call(1.0)])

    @patch('time.sleep')
    def test_server_error_retry(self, time_sleep):
        res = [
            (500, {}, '{}'),
            (500, {}, '{}'),
            (500, {}, '{}'),
            (200, {}, json.dumps({ 'data': 'me' }))
        ]
        responses.add_callback(responses.GET, 'http://app/users/me', callback=lambda r: res.pop(0), content_type='application/json')

        self.assertRaises(asana.error.ServerError, self.client.users.me, max_retries=2)
        self.assertEqual(time_sleep.mock_calls, [call(1.0), call(2.0)])

    @patch('time.sleep')
    def test_server_error_retry_backoff(self, time_sleep):
        res = [
            (500, {}, '{}'),
            (500, {}, '{}'),
            (500, {}, '{}'),
            (200, {}, json.dumps({ 'data': 'me' }))
        ]
        responses.add_callback(responses.GET, 'http://app/users/me', callback=lambda r: res.pop(0), content_type='application/json')

        self.assertEqual(self.client.users.me(), 'me')
        self.assertEqual(time_sleep.mock_calls, [call(1.0), call(2.0), call(4.0)])


    def test_get_named_parameters(self):
        responses.add(GET, 'http://app/tasks?workspace=14916&assignee=me', status=200, body=json.dumps({ 'data': 'dummy data' }), match_querystring=True)

        self.assertEqual(self.client.tasks.find_all(workspace=14916, assignee="me"), 'dummy data')

    def test_post_named_parameters(self):
        responses.add(POST, 'http://app/tasks', status=201, body='{ "data": "dummy data" }', match_querystring=True)

        self.assertEqual(self.client.tasks.create(assignee=1235, followers=[5678], name="Hello, world."), 'dummy data')
        self.assertEqual(json.loads(responses.calls[0].request.body), {
            "data": {
                "assignee": 1235,
                "followers": [5678],
                "name": "Hello, world."
            }
        })

    def test_put_named_parameters(self):
        responses.add(PUT, 'http://app/tasks/1001', status=200, body='{ "data": "dummy data" }', match_querystring=True)

        self.assertEqual(self.client.tasks.update(1001, assignee=1235, followers=[5678], name="Hello, world."), 'dummy data')
        self.assertEqual(json.loads(responses.calls[0].request.body), {
            "data": {
                "assignee": 1235,
                "followers": [5678],
                "name": "Hello, world."
            }
        })




from .helpers import *
import six
import cgi

class TestClientAttachments(ClientTestCase):

    def test_attachments_find_by_id(self):
        res = {
            "data": {
                "created_at": "",
                "download_url": "https://www.dropbox.com/s/1234567890abcdef/Screenshot.png?dl=1",
                "host": "dropbox",
                "id": 5678,
                "name": "Screenshot.png",
                "parent": { "id": 1337, "name": "My Task" },
                "view_url": "https://www.dropbox.com/s/1234567890abcdef/Screenshot.png"
            }
        }
        responses.add(GET, 'http://app/attachments/5678', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.attachments.find_by_id(5678), res['data'])

    def test_attachments_find_by_task(self):
        res = {
            "data": [
                { "id": 5678, "name": "Background.png" },
                { "id": 9012, "name": "New Design Draft.pdf" }
            ]
        }
        responses.add(GET, 'http://app/tasks/1234/attachments', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.attachments.find_by_task(1234), res['data'])

    def test_attachments_create_on_task(self):
        res = { "data": { "id": 5678, "name": "file.txt" } }
        responses.add(POST, 'http://app/tasks/1337/attachments', status=200, body=json.dumps(res), match_querystring=True)

        self.assertEqual(self.client.attachments.create_on_task(1337, 'file content', 'file name', 'file content-type'), res['data'])

        request_content_type, pdict = cgi.parse_header(responses.calls[0].request.headers['Content-Type'])
        self.assertEqual(request_content_type, 'multipart/form-data')

        content_file = six.BytesIO(responses.calls[0].request.body)
        multipart = cgi.parse_multipart(content_file, { 'boundary': six.b(pdict['boundary']) })
        self.assertEqual(multipart['file'][0], six.b('file content'))
        # TODO: verify filename and content-type, possibly using a different multipart decoder




class _Teams:
    """A _team_ is used to group related projects and people together within an
    organization. Each project in an organization is associated with a team.
    """

    def __init__(self, client=None):
        self.client = client
  
    def find_by_id(self, team, params={}, **options): 
        """Returns the full record for a single team.

        Parameters
        ----------
        team : {Id} Globally unique identifier for the team.
        [params] : {Object} Parameters for the request
        """
        path = "/teams/%d" % (team)
        return self.client.get(path, params, **options)
        
    def find_by_organization(self, organization, params={}, **options): 
        """Returns the compact records for all teams in the organization visible to
        the authorized user.

        Parameters
        ----------
        organization : {Id} Globally unique identifier for the workspace or organization.
        [params] : {Object} Parameters for the request
        """
        path = "/organizations/%d/teams" % (organization)
        return self.client.get_collection(path, params, **options)
        
    def users(self, team, params={}, **options): 
        """Returns the compact records for all users that are members of the team.

        Parameters
        ----------
        team : {Id} Globally unique identifier for the team.
        [params] : {Object} Parameters for the request
        """
        path = "/teams/%d/users" % (team)
        return self.client.get_collection(path, params, **options)
        



#!/usr/bin/env python

import sys
import os
from setuptools import setup, find_packages

assert sys.version_info >= (2, 6), 'We only support Python 2.6+'

sys.path.insert(0, os.path.join(os.path.dirname(__file__), 'asana'))
from version import VERSION

setup(name='asana',
      version=VERSION,
      description='Asana API client',
      # license='',
      install_requires=['requests','requests_oauthlib','six'],
      author='Asana, Inc',
      # author_email='',
      url='http://github.com/asana/python-asana',
      packages = find_packages(),
      keywords= 'asana',
      zip_safe = True,
      test_suite='tests')




import asana
import requests
import responses
import unittest
import json

from six import next
from responses import GET, PUT, POST, DELETE

# Define JSON primitives so we can just copy in JSON:
false = False
true = True
null = None

# From https://github.com/dropbox/responses/issues/31#issuecomment-63165210
from inspect import getmembers, isfunction, ismethod
def decallmethods(decorator, prefix='test_'):
    def dectheclass(cls):
        for name, m in getmembers(cls, predicate=lambda x: isfunction(x) or ismethod(x)):
            if name.startswith(prefix):
                setattr(cls, name, decorator(m))
        return cls
    return dectheclass

# TestCase subclass that automatically decorates test methods with responses.activate and sets up a client instance
class ClientTestCase(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        decallmethods(responses.activate)(cls)

    def setUp(self):
        self.client = asana.Client(
            base_url='http://app',
            poll_interval=0, # no delay when polling to speed up tests
            iterator_type=None, # disable iterator and limit to match existing tests for now
        )
